@@ -1,7 +1,9 @@
+from google.auth import transport
 import google.cloud.logging
 import google.auth
 import threading
 import time, os, random
+import google.oauth2.id_token
 
 from uuid import uuid4
 from falcon import Request, Response
@@ -14,6 +16,27 @@ client = google.cloud.logging.Client()
 logger = client.logger(SERVICE)
 
 
+def generate_token(service_url) -> str:
+    """
+    Genarate id token for authentication service to service using container Service account
+    """
+    auth_req = transport.requests.Request()
+    id_token = google.oauth2.id_token.fetch_id_token(auth_req, service_url)
+
+    return id_token
+
+
+def get_default_headers(service_url):
+    """
+    Generate Dictionary header with Authentication param and X-Cloud-Trace-Context param
+    for continuous tracing
+    """
+    return {
+        u'Authorization': u'Bearer {id_token}'.format(id_token=generate_token(service_url)),
+        u'X-Cloud-Trace-Context': u'{}'.format(logging.get_trace_context()),
+    }
+
+
 class _Context:
     def __init__(self):
         self._thread_local = threading.local()
@@ -30,6 +53,10 @@ class _Context:
     def trace_context(self):
         return getattr(self._thread_local, 'trace_context', None)
 
+    @property
+    def endpoint_api_userinfo(self):
+        return getattr(self._thread_local, 'endpoint_api_userinfo', None)
+
     @request_id.setter
     def request_id(self, value):
         self._thread_local.request_id = value
@@ -41,6 +68,10 @@ class _Context:
     @trace_context.setter
     def trace_context(self, value):
         self._thread_local.trace_context = value
+    
+    @endpoint_api_userinfo.setter
+    def trace_context(self, value):
+        self._thread_local.endpoint_api_userinfo = value
 
 
 ctx = _Context
@@ -59,6 +90,8 @@ def set_request_id(request: Request, response: Response):
     
     ctx.trace = f"projects/{PROJECT_ID}/traces/{ctx.request_id}"
     ctx.trace_context = cloud_trace_context
+
+    ctx.endpoint_api_userinfo = request.get_header('X-ENDPOINT-API-USERINFO', None)
     
     request.context.cloud_trace_context = cloud_trace_context
     request.context.trace = ctx.trace
@@ -75,7 +108,9 @@ class LogMiddleware(object):
 
         set_request_id(request, response)
 
-        logger.log_struct({"message": f"request stated"},
+        host = request.host
+
+        logger.log_struct({"message": f"request stated by {host} to service {SERVICE if SERVICE else 'unknown'}"},
             trace=request.context.trace,
             severity="INFO",
             http_request={
@@ -128,6 +163,14 @@ class logging():
     @staticmethod
     def get_trace_context() -> str:
         return ctx.trace_context
+    
+    @staticmethod
+    def get_auth_token(service_url: str) -> str:
+        return generate_token(service_url)
+
+    @staticmethod
+    def get_auth_header(service_url:str) -> dict:
+        return (get_default_headers(service_url), ctx.endpoint_api_userinfo)
 
     @staticmethod
     def info(message: dict):
